function [erxeul,erxpeul,erxrk2,erxprk2,erxrk4,erxprk4]=...
         trace_finalexo3(k,c,f,omega,x0,xp0,N,T)

% trace_finalexo3 : calcul et trac de la solution d'une qua diff par diffrents schmas
% 
%
% *********************************************************
%
%     trace_finalexo3(k,c,f,omega,x0,xp0,N,T)  : calcul et trac de la solution 
%         exacte et approch (par Euler explicite et Runge Kutta 2 et 4) 
%         de la solution de l'quation diffrentielle :
%
%   x''(t)+cx'(t)+kx(t)=f*cos(omega*t), sur [0,T]
%            avec x(0)=x0 et x'(0)=xp0
%
%	
%	* variables d'entres :
%      k,c,f,omega,x0 et xp0 dfinis ci-dessus
%      N : nombre de valeurs calcules ; 
%      T : borne suprieure de l'intervalle ;
%
%	* sortie :
%     Le graphiqueY
%     erxeul et erxpeul : erreur respective entre x et x' et la solution exacte (Euler)
%     erxrk2 et erxprk2 : erreur respective entre x et x' et la solution exacte (RK2)
%     erxrk4 et erxprk4 : erreur respective entre x et x' et la solution exacte (RK4)
% 
% 
% ************ Fonctions auxiliaires utilises ************
%
%   solution_exacte_01, solution_exactep_01
%
% *********************************************************
%


%nombre de points pour l'affichage dfini par ngra 
ngra=1000;


% dfinition de la fonction F (en chane) telle que le systme soit quivalent
%  X'(t)=F(t,X(t)) dans R^2
% F est defini en fonction de t et de X, vecteur  2 composantes.
F='[X(2);f*cos(omega*t)-k*X(1)-c*X(2)]';
h=T/N;

% calcul de la solution par Euler (explicite).
Xeuler=zeros(2,N+1);
Xeuler(:,1)=[x0;xp0];
auxiX=[x0;xp0];
for i=1:N
   t=h*(i-1);
   X=auxiX;
   auxiX=auxiX+h*eval(F);
   Xeuler(:,i+1)=auxiX;
end

% calcul de la solution par Runge Kutta 2.
Xrungekutta2=zeros(2,N+1);
Xrungekutta2(:,1)=[x0;xp0];
auxiX=[x0;xp0];
for i=1:N
   t=h*(i-1);
   X=auxiX;
   K1=h*eval(F);
   t=t+h;
   X=auxiX+K1;
   K2=h*eval(F);
   auxiX=auxiX+(K1+K2)/2;
   Xrungekutta2(:,i+1)=auxiX;
end

% calcul de la solution par Runge Kutta 4.
Xrungekutta4=zeros(2,N+1);
Xrungekutta4(:,1)=[x0;xp0];
auxiX=[x0;xp0];
for i=1:N
   t=h*(i-1);
   X=auxiX;
   K1=h*eval(F);
   t=t+h/2;
   X=auxiX+K1/2;
   K2=h*eval(F);
   X=auxiX+K2/2;
   K3=h*eval(F);
   t=h*i;
   X=auxiX+K3;
   K4=h*eval(F);
   auxiX=auxiX+(K1+2*K2+2*K3+K4)/6;
   Xrungekutta4(:,i+1)=auxiX;
end

% calcul des solutions exactes.
Xexact=zeros(2,N+1);
Xexact(1,:)=real(solution_exacte_01(k,c,f,omega,x0,xp0,0:h:T));
Xexact(2,:)=real(solution_exactep_01(k,c,f,omega,x0,xp0,0:h:T));

% calcul des erreurs.
erxeul=max(abs(Xeuler(1,:)-Xexact(1,:)));
erxpeul=max(abs(Xeuler(2,:)-Xexact(2,:)));
erxrk2=max(abs(Xrungekutta2(1,:)-Xexact(1,:)));
erxprk2=max(abs(Xrungekutta2(2,:)-Xexact(2,:)));
erxrk4=max(abs(Xrungekutta4(1,:)-Xexact(1,:)));
erxprk4=max(abs(Xrungekutta4(2,:)-Xexact(2,:)));

% prparation des vecteurs de trac.
if (N<ngra)
    xx=0:h:T;
    Xexactgra=Xexact;
    Xeulergra=Xeuler;
    Xrungekutta2gra=Xrungekutta4;
    Xrungekutta4gra=Xrungekutta4;
else
    p=fix(N/ngra);
    Xexactgra=[Xexact(:,1:p:N+1) Xexact(:,N+1)];
    Xeulergra=[Xeuler(:,1:p:N+1) Xeuler(:,N+1)];
    Xrungekutta2gra=[Xrungekutta2(:,1:p:N+1) Xrungekutta2(:,N+1)];
    Xrungekutta4gra=[Xrungekutta4(:,1:p:N+1) Xrungekutta4(:,N+1)];
    xx=0:h:T;
    xx=[xx(1:p:N+1) T];
end

% trac
clf;
subplot(2,1,1); 
plot(xx,Xeulergra(1,:),'r',xx,Xrungekutta2gra(1,:),'b',xx,Xrungekutta4gra(1,:),'g',...
    xx,Xexactgra(1,:),'m',[0 T],[0 0],'k');
title('Fonction x en fonction du temps');
legend('Euler','Runge-Kutta 2','Runge-Kutta 4','exacte',0);
subplot(2,1,2); 
plot(xx,Xeulergra(2,:),'r',xx,Xrungekutta2gra(2,:),'b',xx,Xrungekutta2gra(2,:),'g',...
    xx,Xexactgra(2,:),'m', [0 T],[0 0],'k');
legend('Euler','Runge-Kutta 2','Runge-Kutta 4','exacte',0);
title('Fonction x'' en fonction du temps');
diary off
